@@ -3,7 +3,7 @@
 import sys
 
 from .client import WSPipelines
-from .cpd_paths import CpdScope, CpdPath
+from .cpd_paths import CpdScope, CpdPath, CpdScopeFile
 from .version import __version__
 
 if sys.version_info[0] == 2:
