@@ -5,7 +5,7 @@ import io
 import json
 import os
 import warnings
-from abc import abstractmethod
+from abc import abstractmethod, ABC
 from collections import abc
 from enum import Enum
 from functools import wraps
@@ -525,6 +525,26 @@ class WSPipelines(BaseService):
         validate_type(cpd_scope, name, CpdScope)
         return cpd_scope
 
+    def _default_path_to_result(self, result_name: str) -> str:
+        return f'.ibm_watson_studio_pipelines/results/{result_name}'
+
+    def _store_results_via_client(
+        self,
+        storage_client: 'StorageClient',
+        outputs: Mapping[str, Any], # output name -> value
+        output_artifacts: Optional[Mapping[str, str]] = None,
+    ) -> DetailedResponse:
+        if output_artifacts is None:
+            output_artifacts = {
+                out: self._default_path_to_result(out) for out in outputs.keys()
+            }
+
+        response = None
+        for output_name, output_value in outputs.items():
+            result_key = output_artifacts[output_name]
+            response = storage_client.store_result(output_name, result_key, output_value)
+        return response
+
     def store_results(
         self,
         outputs: Mapping[str, Any], # output name -> value
@@ -540,8 +560,7 @@ class WSPipelines(BaseService):
         except MissingValueError:
             test_mode = True
             output_artifacts = {
-                out: f'.ibm_watson_studio_pipelines/results/{out}'
-                for out in outputs.keys()
+                out: self._default_path_to_result(out) for out in outputs.keys()
             }
 
         if test_mode:
@@ -567,12 +586,7 @@ class WSPipelines(BaseService):
                 print(f'    - "{out_name}": {out_path}')
             storage_client = LocalFileSystemClient(self)
 
-        response = None
-        for output_name, output_value in outputs.items():
-            result_key = output_artifacts[output_name]
-            response = storage_client.store_result(output_name, result_key, output_value)
-
-        return response
+        return self._store_results_via_client(storage_client, outputs, output_artifacts)
 
 @attrs(auto_attribs=True, frozen=True)
 class WmlCredentialsAuth:
@@ -686,18 +700,7 @@ class StorageConfig:
         validate_type(auth_endpoint, "auth_endpoint", str)
         return auth_endpoint
 
-class StorageClient(Protocol):
-    @abstractmethod
-    def store_result(self, output_name: str, output_key: str, value: Any) -> DetailedResponse: ...
-
-class LocalFileSystemClient:
-    def __init__(
-        self,
-        cpd_orchestration: WSPipelines
-    ):
-        validate_type(cpd_orchestration, "cpd_orchestration", WSPipelines)
-        self.cpd_orchestration = cpd_orchestration
-
+class StorageClient(ABC):
     def store_result(self, output_name: str, output_key: str, value: Any) -> DetailedResponse:
         validate_type(output_name, "output_name", str)
         validate_type(output_key, "output_key", str)
@@ -705,9 +708,25 @@ class LocalFileSystemClient:
         if isinstance(value, io.TextIOBase):
             # not supported yet
             raise FilesResultsNotSupportedError(output_name)
+        elif isinstance(value, str):
+            str_value = value
         else:
-            value = str(value)
+            str_value = json.dumps(value)
 
+        return self._store_str_result(output_name, output_key, str_value)
+
+    @abstractmethod
+    def _store_str_result(self, output_name: str, output_key: str, value: str) -> DetailedResponse: ...
+
+class LocalFileSystemClient(StorageClient):
+    def __init__(
+        self,
+        cpd_orchestration: WSPipelines
+    ):
+        validate_type(cpd_orchestration, "cpd_orchestration", WSPipelines)
+        self.cpd_orchestration = cpd_orchestration
+
+    def _store_str_result(self, output_name: str, output_key: str, value: str) -> DetailedResponse:
         path = Path(output_key)
 
         status = 201 # created a new file
@@ -729,7 +748,7 @@ class LocalFileSystemClient:
             response=response
         )
 
-class CosClient:
+class CosClient(StorageClient):
     def __init__(
         self,
         cpd_orchestration: WSPipelines,
@@ -749,16 +768,7 @@ class CosClient:
             endpoint_url=config.endpoint
         )
 
-    def store_result(self, output_name: str, output_key: str, value: Any) -> DetailedResponse:
-        validate_type(output_name, "output_name", str)
-        validate_type(output_key, "output_key", str)
-
-        if isinstance(value, io.TextIOBase):
-            # not supported yet
-            raise FilesResultsNotSupportedError(output_name)
-        else:
-            value = str(value)
-
+    def _store_str_result(self, output_name: str, output_key: str, value: str) -> DetailedResponse:
         cos_response = self.cos.Object(self.config.bucket_name, output_key).put(
             Body=value
         )
@@ -774,7 +784,7 @@ class CosClient:
             response=response
         )
 
-class CamsClient:
+class CamsClient(StorageClient):
     def __init__(
         self,
         cpd_orchestration: WSPipelines,
@@ -789,16 +799,7 @@ class CamsClient:
         self.scope = scope
         self.guid = guid
 
-    def store_result(self, output_name: str, output_key: str, value: Any) -> DetailedResponse:
-        validate_type(output_name, "output_name", str)
-        validate_type(output_key, "output_key", str)
-
-        if isinstance(value, io.TextIOBase):
-            # not supported yet
-            raise FilesResultsNotSupportedError(output_name)
-        else:
-            value = str(value)
-
+    def _store_str_result(self, output_name: str, output_key: str, value: str) -> DetailedResponse:
         scope_key = self.scope.scope_type()
         if scope_key.endswith('s'):
             scope_key = scope_key[:-1]
